@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # gPodder - A media aggregator and podcast client
-# Copyright (C) 2005-2007 Thomas Perl <thp at perli.net>
+# Copyright (c) 2005-2008 Thomas Perl and the gPodder Team
 #
 # gPodder is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,6 +20,9 @@
 # The User-Agent string for downloads
 user_agent = 'gPodder'
 
-# Are we running in GUI or console mode?
-interface_is_gui = False
+# Interface type enums
+(CLI, GUI, MAEMO) = range(3)
+
+# Are we running in GUI, Maemo or console mode?
+interface = CLI
 
