@@ -28,9 +28,13 @@
 
 import feedparser
 
+import string
+import re
 import time
-import gpodder
+import urllib, urlparse
 
+import gpodder
+from gpodder import resolver
 from gpodder.liblogger import log
 
 
@@ -73,6 +77,10 @@ class Cache:
     caching.
     """
 
+    # Supported types, see http://feedvalidator.org/docs/warning/EncodingMismatch.html
+    SUPPORTED_FEED_TYPES = ('application/rss+xml', 'application/atom+xml',
+            'application/rdf+xml', 'application/xml', 'text/xml')
+
     def __init__(self, timeToLiveSeconds=3600):
         """
         Arguments:
@@ -105,6 +113,19 @@ class Cache:
             etag = None
             modified = None
 
+        original_url = url
+        # If we have a username or password, rebuild the url with them included
+        # Note: using a HTTPBasicAuthHandler would be pain because we need to
+        # know the realm. It can be done, but I think this method will work fine
+        if old_channel is not None and (
+                old_channel.username or old_channel.password ):
+            username = urllib.quote(old_channel.username)
+            password = urllib.quote(old_channel.password)
+            auth_string = string.join( [username, password], ':' )
+            url_parts = list(urlparse.urlsplit(url))
+            url_parts[1] = string.join( [auth_string, url_parts[1]], '@' )
+            url = urlparse.urlunsplit(url_parts)
+
         # We know we need to fetch, so go ahead and do it.
         parsed_result = feedparser.parse(url,
                                          agent=self.user_agent,
@@ -112,6 +133,53 @@ class Cache:
                                          etag=etag,
                                          )
 
+        # Sometimes, the status code is not set (ugly feed?)
+        status = parsed_result.get('status', None)
+
+        # 304: Not Modified
+        if status == 304:
+            log('Not Modified: %s', url, sender=self)
+            return (False, None)
+
+        if status == 401:
+            log('HTTP authentication required: %s', original_url, sender=self)
+            return (False, parsed_result)
+
+        content_type = parsed_result.headers.get('content-type', '').lower()
+        # TODO: Also detect OPML feeds and other content types here
+        if parsed_result.version == '':
+            log('%s looks like a webpage - trying feed autodiscovery.', url, sender=self)
+            if not hasattr(parsed_result.feed, 'links'):
+                return (False, None)
+            try:
+                found_alternate_feed = False
+                for link in parsed_result.feed.links:
+                    if hasattr(link, 'type') and hasattr(link, 'href') and hasattr(link, 'rel'):
+                        if link.type in self.SUPPORTED_FEED_TYPES and link.rel == 'alternate':
+                            log('Found alternate feed link: %s', link.href, sender=self)
+                            parsed_result = feedparser.parse(link.href,
+                                                             agent=self.user_agent,
+                                                             modified=modified,
+                                                             etag=etag,
+                                                             )
+                            found_alternate_feed = True
+                            break
+
+                # YouTube etc feed lookup (after the normal link lookup in case
+                # they provide a standard feed discovery mechanism in the future).
+                if not found_alternate_feed:
+                    next = resolver.get_real_channel_url(url)
+
+                    if next is not None:
+                        parsed_result = feedparser.parse(next, agent=self.user_agent, modified=modified, etag=etag)
+                        found_alternate_feed = True
+
+                # We have not found a valid feed - abort here!
+                if not found_alternate_feed:
+                    return (False, None)
+            except:
+                log('Error while trying to get feed URL from webpage', sender=self, traceback=True)
+
         updated = False
         status = parsed_result.get('status', None)
 
