@@ -38,17 +38,16 @@ import threading
 import ConfigParser
 
 if gpodder.interface == gpodder.MAEMO:
-    default_bittorrent_dir = '/media/mmc2/gpodder/torrents'
-    default_download_dir = '/media/mmc2/gpodder/downloads'
+    default_download_dir = '/media/mmc2/gpodder'
 else:
-    default_bittorrent_dir = os.path.expanduser('~/gpodder-downloads/torrents')
     default_download_dir = os.path.expanduser('~/gpodder-downloads')
 
 gPodderSettings = {
     # General settings
-    'player': ( str, 'xdg-open' ),
+    'player': (str, 'default'),
     'videoplayer': (str, 'unspecified'),
-    'opml_url': ( str, 'http://www.gpodder.org/directory.opml' ),
+    'opml_url': (str, 'http://gpodder.org/directory.opml'),
+    'toplist_url': (str, 'http://gpodder.org/toplist.opml'),
     'http_proxy': ( str, '' ),
     'ftp_proxy': ( str, '' ),
     'custom_sync_name': ( str, '{episode.basename}' ),
@@ -57,13 +56,11 @@ gPodderSettings = {
     'max_downloads_enabled': ( bool, False ), 
     'limit_rate': ( bool, False ),
     'limit_rate_value': ( float, 500.0 ),
-    'bittorrent_dir': (str, default_bittorrent_dir),
     'episode_old_age': ( int, 7 ),
 
     # Boolean config flags
     'update_on_startup': ( bool, False ),
     'auto_download_when_minimized': (bool, False),
-    'use_gnome_bittorrent': ( bool, True ),
     'only_sync_not_played': ( bool, False ),
     'proxy_use_environment': ( bool, True ),
     'update_tags': ( bool, False ),
@@ -76,6 +73,7 @@ gPodderSettings = {
     'episode_list_descriptions': (bool, True),
     'show_toolbar': (bool, True),
     'ipod_write_gtkpod_extended': (bool, False),
+    'ipod_purge_old_episodes': (bool, False),
     'mp3_player_delete_played': (bool, False),
     
     # Tray icon and notification settings
@@ -86,10 +84,7 @@ gPodderSettings = {
     'on_quit_ask': (bool, True),
 
     # Bluetooth-related settings
-    'bluetooth_enabled': (bool, False),
-    'bluetooth_ask_always': (bool, True),
-    'bluetooth_ask_never': (bool, False),
-    'bluetooth_device_name': (str, 'No device'),
+    'bluetooth_use_device_address': (bool, False),
     'bluetooth_device_address': (str, '00:00:00:00:00:00'),
     'bluetooth_use_converter': (bool, False),
     'bluetooth_converter': (str, ''),
@@ -100,6 +95,11 @@ gPodderSettings = {
     'device_type': ( str, 'none' ),
     'download_dir': (str, default_download_dir),
 
+    # Playlist Management settings
+    'mp3_player_playlist_file': (str, 'PLAYLISTS/gpodder.m3u'),
+    'mp3_player_playlist_absolute_path': (bool, False),
+    'mp3_player_playlist_win_path': (bool, True),
+
     # Special settings (not in preferences)
     'default_new': ( int, 1 ),
     'use_si_units': ( bool, False ),
@@ -108,26 +108,51 @@ gPodderSettings = {
     'max_episodes_per_feed': (int, 200),
     'mp3_player_use_scrobbler_log': (bool, False),
     'mp3_player_max_filename_length': (int, 100),
-    'show_podcast_url_entry': (bool, True),
+    'show_url_entry_in_podcast_list': (bool, False),
     'maemo_allow_custom_player': (bool, False),
     'rockbox_copy_coverart' : (bool, False),
     'rockbox_coverart_size' : (int, 100),
+    'custom_player_copy_coverart' : (bool, False),
+    'custom_player_coverart_size' : (int, 176),
+    'custom_player_coverart_name' : (str, 'folder.jpg'),
+    'custom_player_coverart_format' : (str, 'JPEG'),
     'experimental_file_naming': (bool, False),
     'podcast_list_icon_size': (int, 32),
     'cmd_all_downloads_complete': (str, ''),
     'cmd_download_complete': (str, ''),
-    
+    'enable_streaming': (bool, False),
+    'max_simulaneous_feeds_updating': (int, 3),
+    'color_updating_feeds': (str, '#7db023'),
+    'log_sqlite': (bool, False),
+
     # Hide the cover/pill from the podcast sidebar when it gets too small
     'podcast_sidebar_save_space': (bool, False),
 
-    # Window and paned positions
-    'main_window_x': ( int, 100 ),
-    'main_window_y': ( int, 100 ),
-    'main_window_width': ( int, 700 ),
-    'main_window_height': ( int, 500 ),
+    # Settings for my.gpodder.org
+    'my_gpodder_username': (str, ''),
+    'my_gpodder_password': (str, ''),
+    'my_gpodder_autoupload': (bool, False),
+
+    # Paned position
     'paned_position': ( int, 200 ),
 }
 
+# Helper function to add window-specific properties (position and size)
+def window_props(config_prefix, x=100, y=100, width=700, height=500):
+    return {
+            config_prefix+'_x': (int, x),
+            config_prefix+'_y': (int, y),
+            config_prefix+'_width': (int, width),
+            config_prefix+'_height': (int, height),
+            config_prefix+'_maximized': (bool, False),
+    }
+
+# Register window-specific properties
+gPodderSettings.update(window_props('main_window', width=700, height=500))
+gPodderSettings.update(window_props('episode_selector', width=600, height=400))
+gPodderSettings.update(window_props('episode_window', width=500, height=400))
+
+
 class Config(dict):
     Settings = gPodderSettings
     
@@ -219,29 +244,38 @@ class Config(dict):
             raise ValueError('%s is not a setting'%name)
 
     def receive_configure_event( self, widget, event, config_prefix):
-        ( x, y, width, height ) = map( lambda x: config_prefix + '_' + x, [ 'x', 'y', 'width', 'height' ])
+        (x, y, width, height, maximized) = map(lambda x: config_prefix + '_' + x, ['x', 'y', 'width', 'height', 'maximized'])
         ( x_pos, y_pos ) = widget.get_position()
         ( width_size, height_size ) = widget.get_size()
-        if not self.__ignore_window_events:
+        if not self.__ignore_window_events and not (hasattr(self, maximized) and getattr(self, maximized)):
             setattr( self, x, x_pos)
             setattr( self, y, y_pos)
             setattr( self, width, width_size)
             setattr( self, height, height_size)
 
+    def receive_window_state(self, widget, event, config_prefix):
+        if hasattr(self, config_prefix+'_maximized'):
+            setattr(self, config_prefix+'_maximized', bool(event.new_window_state & gtk.gdk.WINDOW_STATE_MAXIMIZED))
+
     def enable_window_events(self):
         self.__ignore_window_events = False
 
     def disable_window_events(self):
         self.__ignore_window_events = True
 
-    def connect_gtk_window( self, window, config_prefix = 'main_window'):
-        ( x, y, width, height ) = map( lambda x: config_prefix + '_' + x, [ 'x', 'y', 'width', 'height' ])
+    def connect_gtk_window( self, window, config_prefix, show_window=False):
+        (x, y, width, height, maximized) = map(lambda x: config_prefix + '_' + x, ['x', 'y', 'width', 'height', 'maximized'])
         if set( ( x, y, width, height )).issubset( set( self.Settings)):
             window.resize( getattr( self, width), getattr( self, height))
             window.move( getattr( self, x), getattr( self, y))
             self.disable_window_events()
             util.idle_add(self.enable_window_events)
-            window.connect( 'configure-event', self.receive_configure_event, config_prefix)
+            window.connect('configure-event', self.receive_configure_event, config_prefix)
+            window.connect('window-state-event', self.receive_window_state, config_prefix)
+            if show_window:
+                window.show()
+            if hasattr(self, maximized) and getattr(self, maximized) == True:
+                window.maximize()
         else:
             raise ValueError( 'Missing settings in set: %s' % ', '.join( ( x, y, width, height )))
 
