@@ -20,7 +20,9 @@
 from gpodder import util
 from gpodder import download
 from gpodder import sync
+from gpodder.libgpodder import gl
 from gpodder.liblogger import msg
+from gpodder.dbsqlite import db
 
 from libpodcasts import load_channels
 from libpodcasts import update_channels
@@ -42,7 +44,13 @@ def list_channels():
 def add_channel( url):
     callback_error = lambda s: msg( 'error', s)
 
-    url = util.normalize_feed_url( url)
+    url = util.normalize_feed_url(url)
+
+    channels = load_channels()
+    if url in (c.url for c in channels):
+        msg('error', _('Already added: %s'), urllib.unquote(url))
+        return
+
     try:
         channel = podcastChannel.load(url, create=True)
     except:
@@ -50,13 +58,10 @@ def add_channel( url):
         return
 
     if channel:
-        channels = load_channels()
-        if channel.url in ( c.url for c in channels ):
-            msg( 'error', _('Already added: %s'), urllib.unquote( url))
-            return
-        channels.append( channel)
-        save_channels( channels)
-        msg( 'add', urllib.unquote( url))
+        channels.append(channel)
+        save_channels(channels)
+        db.commit()
+        msg('add', urllib.unquote(url))
     else:
         msg('error', _('Could not add podcast.'))
 
@@ -74,6 +79,7 @@ def del_channel( url):
 
     if len(keep_channels) < len(channels):
         save_channels( keep_channels)
+        db.commit()
     else:
         msg('error', _('Could not remove podcast.'))
 
@@ -84,6 +90,7 @@ def update():
     sys.stdout.flush()
     result = update_channels(callback_error=callback_error)
     print _('done.')
+    db.commit()
     return result
 
 def run():
@@ -103,6 +110,7 @@ def run():
         print _('Downloaded one new episode.')
     else:
         print _('Downloaded %d new episodes.') % new_episodes
+    db.commit()
 
 def sync_device():
     device = sync.open_device()
@@ -132,7 +140,37 @@ def sync_device():
                 episodes_to_sync.append(episode)
         device.add_tracks(episodes_to_sync)
 
+    db.commit()
     if not device.close():
         msg('error', _('Cannot close device.'))
         return False
 
+def sync_stats():
+    size = 0
+    device = sync.open_device()
+    if device is None:
+        msg('error', _('No device configured. Please use the GUI.'))
+        return False
+
+    for channel in load_channels():
+        if not channel.sync_to_devices:
+            continue
+        for episode in channel.get_all_episodes():
+            if episode.was_downloaded(and_exists=True):
+                episode.calculate_filesize()
+                size += episode.length
+    msg('info', _('Free space on device %s: %s') % (gl.get_device_name(), gl.format_filesize(device.get_free_space())))
+    msg('info', _('Size of episodes to sync: %s') % gl.format_filesize(size))
+
+    difference = device.get_free_space() - size
+    if difference < 0:
+        msg('error', _('Need to free at least %s more') % gl.format_filesize(abs(difference)))
+        return False
+    else:
+        msg('info', _('Free space after sync: %s') % gl.format_filesize(abs(difference)))
+
+    if not device.close():
+        msg('error', _('Cannot close device.'))
+        return False
+
+    return True
