@@ -29,6 +29,7 @@ from gpodder.liblogger import log
 from gpodder.libgpodder import gl
 from gpodder import util
 from gpodder import services
+from gpodder import resolver
 import gpodder
 
 import threading
@@ -42,6 +43,11 @@ from xml.sax import saxutils
 
 class DownloadCancelledException(Exception): pass
 
+class gPodderDownloadHTTPError(Exception):
+    def __init__(self, url, error_code, error_message):
+        self.url = url
+        self.error_code = error_code
+        self.error_message = error_message
 
 class DownloadURLOpener(urllib.FancyURLopener):
     version = gpodder.user_agent
@@ -59,6 +65,19 @@ class DownloadURLOpener(urllib.FancyURLopener):
         self.channel = channel
         urllib.FancyURLopener.__init__( self, proxies)
 
+    def http_error_default(self, url, fp, errcode, errmsg, headers):
+        """
+        FancyURLopener by default does not raise an exception when
+        there is some unknown HTTP error code. We want to override
+        this and provide a function to log the error and raise an
+        exception, so we don't download the HTTP error page here.
+        """
+        # The following two lines are copied from urllib.URLopener's
+        # implementation of http_error_default
+        void = fp.read()
+        fp.close()
+        raise gPodderDownloadHTTPError(url, errcode, errmsg)
+
     def prompt_user_passwd( self, host, realm):
         if self.channel.username or self.channel.password:
             log( 'Authenticating as "%s" to "%s" for realm "%s".', self.channel.username, host, realm, sender = self)
@@ -89,6 +108,7 @@ class DownloadThread(threading.Thread):
         self.cancelled = False
         self.start_time = 0.0
         self.speed = _('Queued')
+        self.speed_value = 0
         self.progress = 0.0
         self.downloader = DownloadURLOpener( self.channel)
         self.last_update = 0.0
@@ -103,15 +123,27 @@ class DownloadThread(threading.Thread):
 
     def status_updated( self, count, blockSize, totalSize):
         if totalSize:
-            self.progress = 100.0*float(count*blockSize)/float(totalSize)
             # We see a different "total size" while downloading,
             # so correct the total size variable in the thread
-            if totalSize != self.total_size:
+            if totalSize != self.total_size and totalSize > 0:
                 log('Correcting file size for %s from %d to %d while downloading.', self.url, self.total_size, totalSize, sender=self)
                 self.total_size = totalSize
+            elif totalSize < 0:
+                # The current download has a negative value, so assume
+                # the total size given from the feed is correct
+                totalSize = self.total_size
+            self.progress = 100.0*float(count*blockSize)/float(totalSize)
         else:
             self.progress = 100.0
 
+        # Sanity checks for "progress" in valid range (0..100)
+        if self.progress < 0.0:
+            log('Warning: Progress is lower than 0 (count=%d, blockSize=%d, totalSize=%d)', count, blockSize, totalSize, sender=self)
+            self.progress = 0.0
+        elif self.progress > 100.0:
+            log('Warning: Progress is more than 100 (count=%d, blockSize=%d, totalSize=%d)', count, blockSize, totalSize, sender=self)
+            self.progress = 100.0
+
         self.calculate_speed( count, blockSize)
         if self.last_update < time.time() - (1.0 / self.MAX_UPDATES_PER_SEC):
             services.download_status_manager.update_status( self.download_id, speed = self.speed, progress = self.progress)
@@ -152,6 +184,7 @@ class DownloadThread(threading.Thread):
                 speed = count*blockSize
 
             self.speed = '%s/s' % gl.format_filesize(speed)
+            self.speed_value = speed
 
             if gl.config.limit_rate and speed > gl.config.limit_rate_value:
                 # calculate the time that should have passed to reach
@@ -176,8 +209,17 @@ class DownloadThread(threading.Thread):
                     return
          
                 util.delete_file( self.tempname)
-                self.downloader.retrieve( self.episode.url, self.tempname, reporthook = self.status_updated)
+                (unused, headers) = self.downloader.retrieve( resolver.get_real_download_url(self.url), self.tempname, reporthook = self.status_updated)
+
+                if 'content-type' in headers and headers['content-type'] != self.episode.mimetype:
+                    log('Correcting mime type: %s => %s', self.episode.mimetype, headers['content-type'])
+                    self.episode.mimetype = headers['content-type']
+                    # File names are constructed with regard to the mime type.
+                    self.filename = self.episode.local_filename()
+
                 shutil.move( self.tempname, self.filename)
+                # Get the _real_ filesize once we actually have the file
+                self.episode.length = os.path.getsize(self.filename)
                 self.channel.addDownloadedItem( self.episode)
                 services.download_status_manager.download_completed(self.download_id)
                 
@@ -202,6 +244,12 @@ class DownloadThread(threading.Thread):
                 message = _('An error happened while trying to download <b>%s</b>.') % ( saxutils.escape( self.episode.title), )
                 self.notification( message, title)
             log( 'Error "%s" while downloading "%s": %s', ioe.strerror, self.episode.title, ioe.filename, sender = self)
+        except gPodderDownloadHTTPError, gdhe:
+            if self.notification is not None:
+                title = gdhe.error_message
+                message = _('An error (HTTP %d) happened while trying to download <b>%s</b>.') % ( gdhe.error_code, saxutils.escape( self.episode.title), )
+                self.notification( message, title)
+            log( 'HTTP error %s while downloading "%s": %s', gdhe.error_code, self.episode.title, gdhe.error_message, sender=self)
         except:
             log( 'Error while downloading "%s".', self.episode.title, sender = self, traceback = True)
 
