@@ -55,12 +55,33 @@ if gpodder.interface == gpodder.MAEMO:
 class gPodderLib(object):
     def __init__( self):
         log('Creating gPodderLib()', sender=self)
+        gpodder_dir = os.path.expanduser(os.path.join('~', '.config', 'gpodder'))
         if gpodder.interface == gpodder.MAEMO:
-            gpodder_dir = '/media/mmc2/gpodder/'
             self.osso_c = osso.Context('gpodder_osso_sender', '1.0', False)
-        else:
-            gpodder_dir = os.path.expanduser('~/.config/gpodder/')
-        util.make_directory( gpodder_dir)
+            old_dir = '/media/mmc2/gpodder/'
+            if os.path.exists(os.path.join(old_dir, 'channels.opml')) and not os.path.exists(os.path.join(gpodder_dir, 'channels.opml')):
+                # migrate from old (0.13.0 and earlier) gpodder maemo versions
+                # to the current one by moving config files from mmc2 to $HOME
+                util.make_directory(gpodder_dir)
+                for filename in ('channels.opml', 'database.sqlite', 'gpodder.conf'):
+                    try:
+                        shutil.move(os.path.join(old_dir, filename), os.path.join(gpodder_dir, filename))
+                    except:
+                        log('Cannot move %s from %s to %s!', filename, old_dir, gpodder_dir, sender=self, traceback=True)
+                if os.path.exists(os.path.join(old_dir, 'downloads')):
+                    log('Moving old downloads')
+                    # move old download location to new one
+                    for folder in glob.glob(os.path.join(old_dir, 'downloads', '*')):
+                        try:
+                            shutil.move(folder, os.path.join(old_dir, os.path.basename(folder)))
+                        except:
+                            log('Cannot move %s to %s!', folder, old_dir, sender=self, traceback=True)
+                    try:
+                        os.rmdir(os.path.join(old_dir, 'downloads'))
+                    except:
+                        log('Cannot remove old folder %s!', os.path.join(old_dir, 'downloads'), traceback=True)
+
+        util.make_directory(gpodder_dir)
 
         self.tempdir = gpodder_dir
         self.channel_settings_file = os.path.join(gpodder_dir, 'channelsettings.pickle.db')
@@ -73,7 +94,21 @@ class gPodderLib(object):
             self.migrate_channels_xml()
 
         self.config = config.Config( os.path.join( gpodder_dir, 'gpodder.conf'))
-        util.make_directory(self.config.bittorrent_dir)
+
+        if gpodder.interface == gpodder.MAEMO:
+            # Detect changing of SD cards between mmc1/mmc2 if a gpodder
+            # folder exists there (allow moving "gpodder" between SD cards or USB)
+            # Also allow moving "gpodder" to home folder (e.g. rootfs on SD)
+            if not os.path.exists(self.config.download_dir):
+                log('Downloads might have been moved. Trying to locate them...', sender=self)
+                for basedir in ['/media/mmc1', '/media/mmc2']+glob.glob('/media/usb/*')+['/home/user']:
+                    dir = os.path.join(basedir, 'gpodder')
+                    if os.path.exists(dir):
+                        log('Downloads found in: %s', dir, sender=self)
+                        self.config.download_dir = dir
+                        break
+                    else:
+                        log('Downloads NOT FOUND in %s', dir, sender=self)
 
         # We need to make a seamless upgrade, so by default the video player is not specified
         # so the first time this application is run it will detect this and set it to the same 
@@ -82,8 +117,10 @@ class gPodderLib(object):
         if self.config.videoplayer == 'unspecified':
             self.config.videoplayer = self.config.player	
 
+        self.bluetooth_available = util.bluetooth_available()
+
         self.gpodder_dir = gpodder_dir
-        not db.setup({ 'database': os.path.join(gpodder_dir, 'database.sqlite') })
+        not db.setup({ 'database': os.path.join(gpodder_dir, 'database.sqlite'), 'gl': self })
 
     def migrate_to_sqlite(self, add_callback, status_callback, load_channels, get_localdb):
         """
@@ -143,7 +180,6 @@ class gPodderLib(object):
                         p += p_step_2
                     # flush the localdb updates for this channel
                     status_callback(p, _('Writing changes to database'))
-                    db.commit()
                 else:
                     p += p_step
         else:
@@ -242,7 +278,7 @@ class gPodderLib(object):
     def get_device_name( self):
         if self.config.device_type == 'ipod':
             return _('iPod')
-        elif self.config.device_type == 'filesystem':
+        elif self.config.device_type in ('filesystem', 'mtp'):
             return _('MP3 player')
         else:
             log( 'Warning: Called get_device_name() when no device was selected.', sender = self)
@@ -263,7 +299,7 @@ class gPodderLib(object):
         for ddir in download_dirs:
             if os.path.isdir( ddir):
                 globr = glob.glob( '%s/*' % ( ddir, ))
-                if not globr and ddir != self.config.bittorrent_dir:
+                if not globr:
                     log( 'Stale download directory found: %s', os.path.basename( ddir))
                     try:
                         os.rmdir( ddir)
@@ -307,9 +343,15 @@ class gPodderLib(object):
         
         return True
 
-    def playback_episode( self, channel, episode):
+    def playback_episode(self, episode, stream=False):
+        if stream:
+            # A streamed file acts as if it has been deleted
+            episode.state = db.STATE_DELETED
+            db.save_episode(episode)
+            filename = episode.url
+        else:
+            filename = episode.local_filename()
         db.mark_episode(episode.url, is_played=True)
-        filename = episode.local_filename()
 
         if gpodder.interface == gpodder.MAEMO and not self.config.maemo_allow_custom_player:
             # Use the built-in Nokia Mediaplayer here
@@ -317,7 +359,9 @@ class gPodderLib(object):
             osso_rpc = osso.Rpc(self.osso_c)
             service = 'com.nokia.mediaplayer'
             path = '/com/nokia/mediaplayer'
-            osso_rpc.rpc_run(service, path, service, 'mime_open', ('file://'+filename,))
+            if not '://' in filename:
+                filename = 'file://' + filename
+            osso_rpc.rpc_run(service, path, service, 'mime_open', (filename,))
             return (True, service)
 
         # Determine the file type and set the player accordingly.  
@@ -328,8 +372,11 @@ class gPodderLib(object):
         elif file_type == 'audio':
             player = self.config.player
         else:
-            log('Non-audio or video file type, using xdg-open for %s', filename, sender=self)
-            player = 'xdg-open'
+            player = 'default'
+
+        # we should use the default player or no player is set
+        if player == 'default' or player == '':
+            return (util.gui_open(filename), player)
  
         command_line = shlex.split(util.format_desktop_command(player, filename).encode('utf-8'))
         log( 'Command line: [ %s ]', ', '.join( [ '"%s"' % p for p in command_line ]), sender = self)
@@ -339,29 +386,6 @@ class gPodderLib(object):
             return ( False, command_line[0] )
         return ( True, command_line[0] )
 
-    def invoke_torrent( self, url, torrent_filename, target_filename):
-        db.mark_episode(url, is_played=True)
-
-        if self.config.use_gnome_bittorrent:
-            if util.find_command('gnome-btdownload') is None:
-                log( 'Cannot find "gnome-btdownload". Please install gnome-bittorrent.', sender = self)
-                return False
-
-            command = 'gnome-btdownload "%s" --saveas "%s"' % ( torrent_filename, os.path.join( self.config.bittorrent_dir, target_filename))
-            log( command, sender = self)
-            os.system( '%s &' % command)
-            return True
-        else:
-            # Simply copy the .torrent with a suitable name
-            try:
-                target_filename = os.path.join( self.config.bittorrent_dir, os.path.splitext( target_filename)[0] + '.torrent')
-                shutil.copyfile( torrent_filename, target_filename)
-                return True
-            except:
-                log( 'Torrent copy failed: %s => %s.', torrent_filename, target_filename)
-
-        return False
-
     def ext_command_thread(self, notification, command_line):
         """
         This is the function that will be called in a separate
