@@ -39,6 +39,7 @@ from gpodder import services
 from gpodder import draw
 from gpodder import libtagupdate
 from gpodder import dumbshelve
+from gpodder import resolver
 
 from gpodder.liblogger import log
 from gpodder.libgpodder import gl
@@ -64,19 +65,18 @@ from xml.sax import saxutils
 if gpodder.interface == gpodder.MAEMO:
     ICON_AUDIO_FILE = 'gnome-mime-audio-mp3'
     ICON_VIDEO_FILE = 'gnome-mime-video-mp4'
-    ICON_BITTORRENT = 'qgn_toolb_browser_web'
     ICON_DOWNLOADING = 'qgn_toolb_messagin_moveto'
     ICON_DELETED = 'qgn_toolb_gene_deletebutton'
     ICON_NEW = 'qgn_list_gene_favor'
 else:
     ICON_AUDIO_FILE = 'audio-x-generic'
     ICON_VIDEO_FILE = 'video-x-generic'
-    ICON_BITTORRENT = 'applications-internet'
     ICON_DOWNLOADING = gtk.STOCK_GO_DOWN
     ICON_DELETED = gtk.STOCK_DELETE
     ICON_NEW = gtk.STOCK_ABOUT
 
 
+class HTTPAuthError(Exception): pass
 
 class podcastChannel(object):
     """holds data for a complete channel"""
@@ -86,7 +86,7 @@ class podcastChannel(object):
     fc = cache.Cache()
 
     @classmethod
-    def load(cls, url, create=True):
+    def load(cls, url, create=True, authentication_tokens=None):
         if isinstance(url, unicode):
             url = url.encode('utf-8')
 
@@ -95,7 +95,15 @@ class podcastChannel(object):
             return tmp[0]
         elif create:
             tmp = podcastChannel(url)
-            tmp.update()
+            if authentication_tokens is not None:
+                tmp.username = authentication_tokens[0]
+                tmp.password = authentication_tokens[1]
+            success, error_code = tmp.update()
+            if not success:
+                if error_code == 401:
+                    raise HTTPAuthError
+                else:
+                    return None
             tmp.save()
             db.force_last_new(tmp)
             return tmp
@@ -111,11 +119,24 @@ class podcastChannel(object):
     def update(self):
         (updated, c) = self.fc.fetch(self.url, self)
 
+        if c is None:
+            return ( False, None )
+
+        if c.status == 401:
+            return ( False, 401 )
+
+        if self.url != c.url:
+            log('Updating channel URL from %s to %s', self.url, c.url, sender=self)
+            self.url = c.url
+
+        # update the cover if it's not there
+        self.update_cover()
+
         # If we have an old instance of this channel, and
         # feedcache says the feed hasn't changed, return old
         if not updated:
             log('Channel %s is up to date', self.url)
-            return
+            return ( True, None )
 
         # Save etag and last-modified for later reuse
         if c.headers.get('etag'):
@@ -127,24 +148,42 @@ class podcastChannel(object):
 
         if hasattr(c.feed, 'title'):
             self.title = c.feed.title
+            # Start YouTube-specific title FIX
+            YOUTUBE_PREFIX = 'YouTube :: Videos by'
+            if self.title.startswith(YOUTUBE_PREFIX):
+                self.title = self.title[len(YOUTUBE_PREFIX):] + ' on YouTube'
+            # End YouTube-specific title FIX
         else:
             self.title = self.url
         if hasattr( c.feed, 'link'):
             self.link = c.feed.link
         if hasattr( c.feed, 'subtitle'):
-            self.description = util.remove_html_tags(c.feed.subtitle)
+            self.description = c.feed.subtitle
 
         if hasattr(c.feed, 'updated_parsed') and c.feed.updated_parsed is not None:
             self.pubDate = rfc822.mktime_tz(c.feed.updated_parsed+(0,))
         else:
             self.pubDate = time.time()
         if hasattr( c.feed, 'image'):
-            if c.feed.image.href:
+            if hasattr(c.feed.image, 'href') and c.feed.image.href:
+                old = self.image
                 self.image = c.feed.image.href
+                if old != self.image:
+                    self.update_cover(force=True)
 
         # Marked as bulk because we commit after importing episodes.
         db.save_channel(self, bulk=True)
 
+        # Remove old episodes before adding the new ones.  This helps
+        # deal with hyperactive channels, such as TV news, when there
+        # can be more new episodes than the user wants in the list.
+        # By cleaning up old episodes before receiving the new ones we
+        # ensure that the user doesn't miss any.
+        db.purge(gl.config.max_episodes_per_feed, self.id)
+
+        # Load all episodes to update them properly.
+        existing = self.get_all_episodes()
+
         # We can limit the maximum number of entries that gPodder will parse
         # via the "max_episodes_per_feed" configuration option.
         if len(c.entries) > gl.config.max_episodes_per_feed:
@@ -158,10 +197,26 @@ class podcastChannel(object):
                 log('Cannot instantiate episode "%s": %s. Skipping.', entry.get('id', '(no id available)'), e, sender=self, traceback=True)
 
             if episode:
+                self.count_new += 1
+
+                for ex in existing:
+                    if ex.guid == episode.guid:
+                        for k in ('title', 'title', 'description', 'link', 'pubDate'):
+                            setattr(ex, k, getattr(episode, k))
+                        self.count_new -= 1
+                        episode = ex
+
+                if not episode.length:
+                    episode.length = resolver.get_real_episode_length(episode)
+
                 episode.save(bulk=True)
 
-        # Now we can flush the updates.
-        db.commit()
+        return ( True, None )
+
+    def update_cover(self, force=False):
+        if self.cover_file is None or not os.path.exists(self.cover_file) or force:
+            if self.image is not None:
+                services.cover_downloader.request_cover(self)
 
     def delete(self):
         db.delete_channel(self)
@@ -177,11 +232,13 @@ class podcastChannel(object):
         self.url = url
         self.title = title
         self.link = link
-        self.description = util.remove_html_tags( description)
+        self.description = description
         self.image = None
         self.pubDate = 0
         self.parse_error = None
         self.newest_pubdate_cached = None
+        self.update_flag = False # channel is updating or to be updated
+        self.iter = None
 
         # should this channel be synced to devices? (ex: iPod)
         self.sync_to_devices = True
@@ -198,7 +255,11 @@ class podcastChannel(object):
         self.save_dir_size = 0
         self.__save_dir_size_set = False
 
-        self.__tree_model = None
+        self.count_downloaded = 0
+        self.count_new = 0
+        self.count_unplayed = 0
+
+        self.channel_is_locked = False
 
     def request_save_dir_size(self):
         if not self.__save_dir_size_set:
@@ -243,7 +304,7 @@ class podcastChannel(object):
         db.save_channel(self)
     
     def get_new_episodes( self):
-        return [episode for episode in db.load_episodes(self, factory=lambda x: podcastItem.create_from_dict(x, self)) if episode.state == db.STATE_NORMAL and not episode.is_played]
+        return [episode for episode in db.load_episodes(self, factory=lambda x: podcastItem.create_from_dict(x, self)) if episode.state == db.STATE_NORMAL and not episode.is_played and not services.download_status_manager.is_download_in_progress(episode.url)]
 
     def update_m3u_playlist(self):
         if gl.config.create_m3u_playlists:
@@ -268,51 +329,45 @@ class podcastChannel(object):
         log('addDownloadedItem(%s)', item.url)
 
         if not item.was_downloaded():
-            item.mark(is_played=False, state=db.STATE_DOWNLOADED)
+            item.mark_downloaded(save=True)
 
             # Update metadata on file (if possible and wanted)
             if gl.config.update_tags and libtagupdate.tagging_supported():
                 filename = item.local_filename()
                 try:
-                    libtagupdate.update_metadata_on_file(filename, title=item.title, artist=self.title)
+                    libtagupdate.update_metadata_on_file(filename, title=item.title, artist=self.title, genre='Podcast')
                 except Exception, e:
                     log('Error while calling update_metadata_on_file(): %s', e)
 
             self.update_m3u_playlist()
-            
-            if item.file_type() == 'torrent':
-                torrent_filename = item.local_filename()
-                destination_filename = util.torrent_filename( torrent_filename)
-                gl.invoke_torrent(item.url, torrent_filename, destination_filename)
 
     def get_all_episodes(self):
-        return db.load_episodes(self, factory = lambda d: podcastItem.create_from_dict(d, self), limit=gl.config.max_episodes_per_feed)
-
-    def force_update_tree_model( self):
-        self.__tree_model = None
+        return db.load_episodes(self, factory = lambda d: podcastItem.create_from_dict(d, self))
 
+    # not used anymore
     def update_model( self):
         self.update_save_dir_size()
+        model = self.tree_model
 
-        iter = self.tree_model.get_iter_first()
+        iter = model.get_iter_first()
         while iter is not None:
-            self.iter_set_downloading_columns( self.tree_model, iter)
-            iter = self.tree_model.iter_next( iter)
+            self.iter_set_downloading_columns(model, iter)
+            iter = model.iter_next( iter)
 
     @property
     def tree_model( self):
-        if not self.__tree_model:
-            log('Generating TreeModel for %s', self.url, sender = self)
-            self.__tree_model = self.items_liststore()
+        log('Returning TreeModel for %s', self.url, sender = self)
+        return self.items_liststore()
 
-        return self.__tree_model
-
-    def iter_set_downloading_columns( self, model, iter):
-        global ICON_AUDIO_FILE, ICON_VIDEO_FILE, ICON_BITTORRENT
+    def iter_set_downloading_columns( self, model, iter, episode=None):
+        global ICON_AUDIO_FILE, ICON_VIDEO_FILE
         global ICON_DOWNLOADING, ICON_DELETED, ICON_NEW
         
-        url = model.get_value( iter, 0)
-        episode = db.load_episode(url, factory=lambda x: podcastItem.create_from_dict(x, self))
+        if episode is None:
+            url = model.get_value( iter, 0)
+            episode = db.load_episode(url, factory=lambda x: podcastItem.create_from_dict(x, self))
+        else:
+            url = episode.url
 
         if gl.config.episode_list_descriptions:
             icon_size = 32
@@ -322,18 +377,12 @@ class podcastChannel(object):
         if services.download_status_manager.is_download_in_progress(url):
             status_icon = util.get_tree_icon(ICON_DOWNLOADING, icon_cache=self.icon_cache, icon_size=icon_size)
         else:
-            if episode.state != db.STATE_DOWNLOADED and episode.file_exists():
-                episode.mark(state=db.STATE_DOWNLOADED)
-                log('Resurrected episode %s', episode.guid)
-            elif episode.state == db.STATE_DOWNLOADED and not episode.file_exists():
-                episode.mark(state=db.STATE_DELETED)
-                log('Burried episode %s', episode.guid)
             if episode.state == db.STATE_NORMAL:
                 if episode.is_played:
                     status_icon = None
                 else:
                     status_icon = util.get_tree_icon(ICON_NEW, icon_cache=self.icon_cache, icon_size=icon_size)
-            elif episode.was_downloaded(and_exists=True):
+            elif episode.was_downloaded():
                 missing = not episode.file_exists()
 
                 if missing:
@@ -344,12 +393,10 @@ class podcastChannel(object):
                     status_icon = util.get_tree_icon(ICON_AUDIO_FILE, not episode.is_played, episode.is_locked, not episode.file_exists(), self.icon_cache, icon_size)
                 elif file_type == 'video':
                     status_icon = util.get_tree_icon(ICON_VIDEO_FILE, not episode.is_played, episode.is_locked, not episode.file_exists(), self.icon_cache, icon_size)
-                elif file_type == 'torrent':
-                    status_icon = util.get_tree_icon(ICON_BITTORRENT, not episode.is_played, episode.is_locked, not episode.file_exists(), self.icon_cache, icon_size)
                 else:
                     status_icon = util.get_tree_icon('unknown', not episode.is_played, episode.is_locked, not episode.file_exists(), self.icon_cache, icon_size)
             elif episode.state == db.STATE_DELETED or episode.state == db.STATE_DOWNLOADED:
-                status_icon = util.get_tree_icon(ICON_DELETED, icon_cache=self.icon_cache, icon_size=icon_size)
+                status_icon = util.get_tree_icon(ICON_DELETED, not episode.is_played, icon_cache=self.icon_cache, icon_size=icon_size)
             else:
                 log('Warning: Cannot determine status icon.', sender=self)
                 status_icon = None
@@ -365,20 +412,17 @@ class podcastChannel(object):
             gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING )
 
         for item in self.get_all_episodes():
-            if gl.config.episode_list_descriptions:
-                description = '%s\n<small>%s</small>' % (saxutils.escape(item.title), saxutils.escape(item.one_line_description()))
-            else:
-                description = saxutils.escape(item.title)
+            description = item.title_and_description
 
-            if item.length:
+            if item.length > 0:
                 filelength = gl.format_filesize(item.length, 1)
             else:
                 filelength = None
 
             new_iter = new_model.append((item.url, item.title, filelength, 
-                True, None, item.cute_pubdate(), description, item.description, 
+                True, None, item.cute_pubdate(), description, util.remove_html_tags(item.description), 
                 item.local_filename(), item.extension()))
-            self.iter_set_downloading_columns( new_model, new_iter)
+            self.iter_set_downloading_columns( new_model, new_iter, episode=item)
         
         self.update_save_dir_size()
         return new_model
@@ -441,7 +485,7 @@ class podcastItem(object):
 
         episode.title = entry.get( 'title', util.get_first_line( util.remove_html_tags( entry.get( 'summary', ''))))
         episode.link = entry.get( 'link', '')
-        episode.description = util.remove_html_tags( entry.get( 'summary', entry.get( 'link', entry.get( 'title', ''))))
+        episode.description = entry.get( 'summary', entry.get( 'link', entry.get( 'title', '')))
         episode.guid = entry.get( 'id', '')
         if entry.get( 'updated_parsed', None):
             episode.pubDate = rfc822.mktime_tz(entry.updated_parsed+(0,))
@@ -481,7 +525,7 @@ class podcastItem(object):
                 except:
                     log('Cannot convert pubDate "%s" in from_feedparser_entry.', str(metainfo['pubdate']), traceback=True)
 
-        if hasattr( enclosure, 'length'):
+        if hasattr(enclosure, 'length'):
             try:
                 episode.length = int(enclosure.length)
             except:
@@ -512,7 +556,7 @@ class podcastItem(object):
 
         self.state = db.STATE_NORMAL
         self.is_played = False
-        self.is_locked = False
+        self.is_locked = channel.channel_is_locked
 
     def save(self, bulk=False):
         if self.state != db.STATE_DOWNLOADED and self.file_exists():
@@ -532,6 +576,12 @@ class podcastItem(object):
             self.is_locked = is_locked
         db.mark_episode(self.url, state=state, is_played=is_played, is_locked=is_locked)
 
+    def mark_downloaded(self, save=False):
+        self.state = db.STATE_DOWNLOADED
+        self.is_played = False
+        if save:
+            self.save()
+
     @staticmethod
     def create_from_dict(d, channel):
         e = podcastItem(channel)
@@ -540,6 +590,18 @@ class podcastItem(object):
                 setattr(e, key, d[key])
         return e
 
+    @property
+    def title_and_description(self):
+        """
+        Returns Pango markup for displaying in a TreeView, and
+        disables the description when the config variable
+        "episode_list_descriptions" is not set.
+        """
+        if gl.config.episode_list_descriptions:
+            return '%s\n<small>%s</small>' % (saxutils.escape(self.title), saxutils.escape(self.one_line_description()))
+        else:
+            return saxutils.escape(self.title)
+
     def age_in_days(self):
         return util.file_age_in_days(self.local_filename())
 
@@ -552,7 +614,7 @@ class podcastItem(object):
     age_prop = property(fget=get_age_string)
 
     def one_line_description( self):
-        lines = self.description.strip().splitlines()
+        lines = util.remove_html_tags(self.description).strip().splitlines()
         if not lines or lines[0] == '':
             return _('No description available')
         else:
@@ -614,7 +676,10 @@ class podcastItem(object):
 
     def sync_filename( self):
         if gl.config.custom_sync_name_enabled:
-            return util.object_string_formatter(gl.config.custom_sync_name, episode=self, channel=self.channel)
+            if '{channel' in gl.config.custom_sync_name:
+                log('Fixing OLD syntax {channel.*} => {podcast.*} in custom_sync_name.', sender=self)
+                gl.config.custom_sync_name = gl.config.custom_sync_name.replace('{channel.', '{podcast.')
+            return util.object_string_formatter(gl.config.custom_sync_name, episode=self, podcast=self.channel)
         else:
             return self.title
 
@@ -668,48 +733,62 @@ class podcastItem(object):
     
 
 
+def update_channel_model_by_iter( model, iter, channel, color_dict,
+    cover_cache=None, max_width=0, max_height=0 ):
 
-def channels_to_model(channels, cover_cache=None, max_width=0, max_height=0):
-    new_model = gtk.ListStore(str, str, str, gtk.gdk.Pixbuf, int, gtk.gdk.Pixbuf, str, bool)
+    count_downloaded = channel.stat(state=db.STATE_DOWNLOADED)
+    count_new = channel.stat(state=db.STATE_NORMAL, is_played=False)
+    count_unplayed = channel.stat(state=db.STATE_DOWNLOADED, is_played=False)
+
+    channel.iter = iter
+    model.set(iter, 0, channel.url)
+    model.set(iter, 1, channel.title)
+
+    title_markup = saxutils.escape(channel.title)
+    description_markup = saxutils.escape(util.get_first_line(channel.description) or _('No description available'))
+    d = []
+    if count_new:
+        d.append('<span weight="bold">')
+    d.append(title_markup)
+    if count_new:
+        d.append('</span>')
+
+    description = ''.join(d+['\n', '<small>', description_markup, '</small>'])
+    model.set(iter, 2, description)
+
+    if channel.parse_error is not None:
+        model.set(iter, 6, channel.parse_error)
+        color = color_dict['parse_error']
+    else:
+        color = color_dict['default']
+
+    if channel.update_flag:
+        color = color_dict['updating']
+
+    model.set(iter, 8, color)
+
+    if count_unplayed > 0 or count_downloaded > 0:
+        model.set(iter, 3, draw.draw_pill_pixbuf(str(count_unplayed), str(count_downloaded)))
+        model.set(iter, 7, True)
+    else:
+        model.set(iter, 7, False)
+
+    # Load the cover if we have it, but don't download
+    # it if it's not available (to avoid blocking here)
+    pixbuf = services.cover_downloader.get_cover(channel, avoid_downloading=True)
+    new_pixbuf = None
+    if pixbuf is not None:
+        new_pixbuf = util.resize_pixbuf_keep_ratio(pixbuf, max_width, max_height, channel.url, cover_cache)
+    model.set(iter, 5, new_pixbuf or pixbuf)
+
+def channels_to_model(channels, color_dict, cover_cache=None, max_width=0, max_height=0):
+    new_model = gtk.ListStore( str, str, str, gtk.gdk.Pixbuf, int,
+        gtk.gdk.Pixbuf, str, bool, str )
 
     for channel in channels:
-        count_downloaded = channel.stat(state=db.STATE_DOWNLOADED)
-        count_new = channel.stat(state=db.STATE_NORMAL, is_played=False)
-        count_unplayed = channel.stat(state=db.STATE_DOWNLOADED, is_played=False)
+        update_channel_model_by_iter( new_model, new_model.append(), channel,
+            color_dict, cover_cache, max_width, max_height )
 
-        new_iter = new_model.append()
-        new_model.set(new_iter, 0, channel.url)
-        new_model.set(new_iter, 1, channel.title)
-
-        title_markup = saxutils.escape(channel.title)
-        description_markup = saxutils.escape(util.get_first_line(channel.description) or _('No description available'))
-        d = []
-        if count_new:
-            d.append('<span weight="bold">')
-        d.append(title_markup)
-        if count_new:
-            d.append('</span>')
-        description = ''.join(d+['\n', '<small>', description_markup, '</small>'])
-        if channel.parse_error is not None:
-            description = ''.join(['<span foreground="#ff0000">', description, '</span>'])
-            new_model.set(new_iter, 6, channel.parse_error)
-        
-        new_model.set(new_iter, 2, description)
-
-        if count_unplayed > 0 or count_downloaded > 0:
-            new_model.set(new_iter, 3, draw.draw_pill_pixbuf(str(count_unplayed), str(count_downloaded)))
-            new_model.set(new_iter, 7, True)
-        else:
-            new_model.set(new_iter, 7, False)
-
-        # Load the cover if we have it, but don't download
-        # it if it's not available (to avoid blocking here)
-        #pixbuf = services.cover_downloader.get_cover(channel, avoid_downloading=True)
-        #new_pixbuf = None
-        #if pixbuf is not None:
-        #    new_pixbuf = util.resize_pixbuf_keep_ratio(pixbuf, max_width, max_height, channel.url, cover_cache)
-        #new_model.set(new_iter, 5, new_pixbuf or pixbuf)
-    
     return new_model
 
 
