@@ -90,10 +90,16 @@ def update_tag_mp3( filename, **metadata):
     for key in metadata:
         if key.lower() == 'artist':
             tag.setArtist( metadata[key])
+            # Set "album" info from artist name if it is not available
+            if 'album' not in metadata and tag.getAlbum().strip() == '':
+                log('(tagupdate) No album information - using artist')
+                tag.setAlbum(metadata[key])
         elif key.lower() == 'title':
             tag.setTitle( metadata[key])
         elif key.lower() == 'album':
             tag.setAlbum( metadata[key])
+        elif key.lower() == 'genre':
+            tag.setGenre(metadata[key])
 
     return tag.update( eyeD3.tag.ID3_V2) == 1 and tag.update( eyeD3.tag.ID3_V1) == 1
 
