@@ -28,11 +28,13 @@ from gpodder.liblogger import log
 from gpodder.libgpodder import gl
 
 from gpodder import util
+from gpodder import resolver
 
 import gtk
 import gobject
 
 import threading
+import time
 import urllib2
 import os
 import os.path
@@ -70,6 +72,85 @@ class ObservableService(object):
             log('Signal "%s" is not available for notification.', signal_name, sender=self)
 
 
+class DependencyManager(object):
+    def __init__(self):
+        self.dependencies = []
+
+    def depend_on(self, feature_name, description, modules, tools):
+        self.dependencies.append([feature_name, description, modules, tools])
+
+    def modules_available(self, modules):
+        """
+        Receives a list of modules and checks if each
+        of them is available. Returns a tuple with the
+        first item being a boolean variable that is True
+        when all required modules are available and False
+        otherwise. The second item is a dictionary that
+        lists every module as key with the available as
+        boolean value.
+        """
+        result = {}
+        all_available = True
+        for module in modules:
+            try:
+                __import__(module)
+                result[module] = True
+            except:
+                result[module] = False
+                all_available = False
+
+        return (all_available, result)
+
+    def tools_available(self, tools):
+        """
+        See modules_available.
+        """
+        result = {}
+        all_available = True
+        for tool in tools:
+            if util.find_command(tool):
+                result[tool] = True
+            else:
+                result[tool] = False
+                all_available = False
+
+        return (all_available, result)
+
+    def get_model(self):
+        # Name, Description, Available (str), Available (bool), Missing (str)
+        model = gtk.ListStore(str, str, str, bool, str)
+        for feature_name, description, modules, tools in self.dependencies:
+            modules_available, module_info = self.modules_available(modules)
+            tools_available, tool_info = self.tools_available(tools)
+
+            available = modules_available and tools_available
+            if available:
+                available_str = _('Available')
+            else:
+                available_str = _('Missing dependencies')
+
+            missing_str = []
+            for module in modules:
+                if not module_info[module]:
+                    missing_str.append(_('Python module "%s" not installed') % module)
+            for tool in tools:
+                if not tool_info[tool]:
+                    missing_str.append(_('Command "%s" not installed') % tool)
+            missing_str = '\n'.join(missing_str)
+
+            model.append([feature_name, description, available_str, available, missing_str])
+        return model
+
+
+dependency_manager = DependencyManager()
+
+
+# Register non-module-specific dependencies here
+dependency_manager.depend_on(_('Bluetooth file transfer'), _('Send podcast episodes to Bluetooth devices. Needs Python Bluez bindings.'), ['bluetooth'], ['bluetooth-sendto'])
+dependency_manager.depend_on(_('Update tags on MP3 files'), _('Support the "Update tags after download" option for MP3 files.'), ['eyeD3'], [])
+dependency_manager.depend_on(_('Update tags on OGG files'), _('Support the "Update tags after download" option for OGG files.'), [], ['vorbiscomment'])
+
+
 class CoverDownloader(ObservableService):
     """
     This class manages downloading cover art and notification
@@ -100,6 +181,7 @@ class CoverDownloader(ObservableService):
         be downloaded from the specified URL and not
         taken from the channel metadata.
         """
+        log('cover download request for %s', channel.url, sender=self)
         args = [channel, custom_url, True]
         threading.Thread(target=self.__get_cover, args=args).start()
 
@@ -150,6 +232,10 @@ class CoverDownloader(ObservableService):
             if url is None:
                 url = channel.image
 
+            new_url = resolver.get_real_cover(channel.url)
+            if new_url is not None:
+                url = new_url
+
             if url is not None:
                 image_data = None
                 try:
@@ -166,8 +252,8 @@ class CoverDownloader(ObservableService):
                     fp.close()
 
         if os.path.exists(channel.cover_file):
-            loader.write(open(channel.cover_file, 'rb').read())
             try:
+                loader.write(open(channel.cover_file, 'rb').read())
                 loader.close()
                 pixbuf = loader.get_pixbuf()
             except:
@@ -197,12 +283,14 @@ cover_downloader = CoverDownloader()
 class DownloadStatusManager(ObservableService):
     COLUMN_NAMES = { 0: 'episode', 1: 'speed', 2: 'progress', 3: 'url' }
     COLUMN_TYPES = ( gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_FLOAT, gobject.TYPE_STRING )
+    PROGRESS_HOLDDOWN_TIMEOUT = 1
 
     def __init__( self):
         self.status_list = {}
         self.next_status_id = 0
 
-        self.last_progress_status  = ( 0, 0 )
+        self.last_progress_status = (0, 0)
+        self.last_progress_update = 0
         
         # use to correctly calculate percentage done
         self.downloads_done_bytes = 0
@@ -249,12 +337,16 @@ class DownloadStatusManager(ObservableService):
             self.notify('list-changed')
         self.batch_mode_notify_flag = False
 
-    def notify_progress( self):
-        now = ( self.count(), self.average_progress() )
+    def notify_progress(self, force=False):
+        now = (self.count(), self.average_progress())
         
-        if now != self.last_progress_status:
+        next_progress_update = self.last_progress_update + self.PROGRESS_HOLDDOWN_TIMEOUT
+
+        if force or (now != self.last_progress_status and \
+                time.time() > next_progress_update):
             self.notify( 'progress-changed', *now)
             self.last_progress_status = now
+            self.last_progress_update = time.time()
 
     def s_acquire( self):
         if not gl.config.max_downloads_enabled:
@@ -335,7 +427,7 @@ class DownloadStatusManager(ObservableService):
             self.batch_mode_notify_flag = True
         else:
             self.notify('list-changed')
-        self.notify_progress()
+        self.notify_progress(force=True)
 
     def count( self):
         return len(self.status_list)
@@ -353,6 +445,12 @@ class DownloadStatusManager(ObservableService):
             return 0
         return float(done + self.downloads_done_bytes) / float(total + self.downloads_done_bytes) * 100
 
+    def total_speed(self):
+        if not len(self.status_list):
+            return 0
+
+        return sum(status['thread'].speed_value for status in self.status_list.values())
+
     def update_status( self, id, **kwargs):
         if not id in self.status_list:
             return
@@ -397,19 +495,19 @@ class DownloadStatusManager(ObservableService):
 
     def cancel_all( self):
         for element in self.status_list:
-	    self.status_list[element]['iter'] = None
-	    self.status_list[element]['thread'].cancel()
+            self.status_list[element]['iter'] = None
+            self.status_list[element]['thread'].cancel()
         # clear the tree model after cancelling
         util.idle_add(self.tree_model.clear)
         self.downloads_done_bytes = 0
 
     def cancel_by_url( self, url):
         for element in self.status_list:
-	    thread = self.status_list[element]['thread']
-	    if thread is not None and thread.url == url:
+            thread = self.status_list[element]['thread']
+            if thread is not None and thread.url == url:
                 self.remove_download_id( element)
-		return True
-        
+                return True
+
         return False
 
 
