@@ -112,11 +112,17 @@ def normalize_feed_url( url):
     This will also normalize feed:// and itpc:// to http://
     Also supported are phobos.apple.com links (iTunes podcast)
     and itms:// links (iTunes podcast direct link).
+
+    If no URL scheme is defined (e.g. "curry.com"), we will
+    simply assume the user intends to add a http:// feed.
     """
 
     if not url or len( url) < 8:
         return None
 
+    if not '://' in url:
+        url = 'http://' + url
+
     if url.startswith('itms://'):
         url = parse_itunes_xml(url)
 
@@ -400,30 +406,6 @@ def remove_html_tags(html):
     return result.strip()
 
 
-def torrent_filename( filename):
-    """
-    Checks if a file is a ".torrent" file by examining its 
-    contents and searching for the file name of the file 
-    to be downloaded.
-
-    Returns the name of the file the ".torrent" will download 
-    or None if no filename is found (the file is no ".torrent")
-    """
-    if not os.path.exists( filename):
-        return None
-
-    header = open( filename).readline()
-    try:
-        header.index( '6:pieces')
-        name_length_pos = header.index('4:name') + 6
-
-        colon_pos = header.find( ':', name_length_pos)
-        name_length = int(header[name_length_pos:colon_pos]) + 1
-        name = header[(colon_pos + 1):(colon_pos + name_length)]
-        return name
-    except:
-        return None
-
 def extension_from_mimetype(mimetype):
     """
     Simply guesses what the file extension should be from the mimetype
@@ -437,7 +419,7 @@ def filename_from_url(url):
     will result in the string ("file", ".mp3") being returned.
 
     This function will also try to best-guess the "real" 
-    extension for a media file (audio, video, torrent) by 
+    extension for a media file (audio, video) by
     trying to match an extension to these types and recurse
     into the query string to find better matches, if the 
     original extension does not resolve to a known type.
@@ -451,7 +433,7 @@ def filename_from_url(url):
 
     if file_type_by_extension(extension) is not None and not \
         query.startswith(scheme+'://'):
-        # We have found a valid extension (audio, video, torrent)
+        # We have found a valid extension (audio, video)
         # and the query string doesn't look like a URL
         return ( filename, extension.lower() )
 
@@ -471,13 +453,12 @@ def file_type_by_extension( extension):
     """
     Tries to guess the file type by looking up the filename 
     extension from a table of known file types. Will return 
-    the type as string ("audio", "video" or "torrent") or 
+    the type as string ("audio" or "video") or
     None if the file type cannot be determined.
     """
     types = {
             'audio': [ 'mp3', 'ogg', 'wav', 'wma', 'aac', 'm4a' ],
             'video': [ 'mp4', 'avi', 'mpg', 'mpeg', 'm4v', 'mov', 'divx', 'flv', 'wmv', '3gp' ],
-            'torrent': [ 'torrent' ],
     }
 
     if extension == '':
@@ -611,9 +592,14 @@ def format_desktop_command( command, filename):
 
     See http://standards.freedesktop.org/desktop-entry-spec/1.0/ar01s06.html
     """
+    if '://' in filename:
+        filename_url = filename
+    else:
+        filename_url = 'file://%s' % filename
+
     items = {
-            '%U': 'file://%s' % filename,
-            '%u': 'file://%s' % filename,
+            '%U': filename_url,
+            '%u': filename_url,
             '%F': filename,
             '%f': filename,
     }
@@ -778,6 +764,19 @@ def discover_bluetooth_devices():
             return # <= empty generator
 
 
+def bluetooth_available():
+    """
+    Returns True or False depending on the availability
+    of bluetooth functionality on the system.
+    """
+    if find_command('bluetooth-sendto'):
+        return True
+    elif find_command('gnome-obex-send'):
+        return True
+    else:
+        return False
+
+
 def bluetooth_send_file(filename, device=None, callback_finished=None):
     """
     Sends a file via bluetooth using gnome-obex send.
@@ -859,6 +858,18 @@ def format_seconds_to_hour_min_sec(seconds):
     else:
         return result[0]
 
+def proxy_request(url, proxy=None, method='HEAD'):
+    if proxy is None or proxy.strip() == '':
+        (scheme, netloc, path, parms, qry, fragid) = urlparse.urlparse(url)
+        conn = httplib.HTTPConnection(netloc)
+        start = len(scheme) + len('://') + len(netloc)
+        conn.request(method, url[start:])
+    else:
+        (scheme, netloc, path, parms, qry, fragid) = urlparse.urlparse(proxy)
+        conn = httplib.HTTPConnection(netloc)
+        conn.request(method, url)
+
+    return conn.getresponse()
 
 def get_episode_info_from_url(url, proxy=None):
     """
@@ -880,17 +891,7 @@ def get_episode_info_from_url(url, proxy=None):
     if not (url.startswith('http://') or url.startswith('https://')):
         return {}
 
-    if proxy is None or proxy.strip() == '':
-        (scheme, netloc, path, parms, qry, fragid) = urlparse.urlparse(url)
-        conn = httplib.HTTPConnection(netloc)
-        start = len(scheme) + len('://') + len(netloc)
-        conn.request('HEAD', url[start:])
-    else:
-        (scheme, netloc, path, parms, qry, fragid) = urlparse.urlparse(proxy)
-        conn = httplib.HTTPConnection(netloc)
-        conn.request('HEAD', url)
-
-    r = conn.getresponse()
+    r = proxy_request(url, proxy)
     result = {}
 
     log('Trying to get metainfo for %s', url)
@@ -922,8 +923,10 @@ def gui_open(filename):
         subprocess.Popen(['xdg-open', filename])
         # FIXME: Win32-specific "open" code needed here
         # as fallback when xdg-open not available
+        return True
     except:
         log('Cannot open file/folder: "%s"', filename, sender=self, traceback=True)
+        return False
 
 
 def open_website(url):
@@ -934,20 +937,36 @@ def open_website(url):
     """
     threading.Thread(target=webbrowser.open, args=(url,)).start()
 
+def sanitize_encoding(filename):
+    """
+    Generate a sanitized version of a string (i.e.
+    remove invalid characters and encode in the
+    detected native language encoding)
+    """
+    global encoding
+    return filename.encode(encoding, 'ignore')
 
-def sanitize_filename(filename, max_length=0):
+
+def sanitize_filename(filename, max_length=0, use_ascii=False):
     """
     Generate a sanitized version of a filename that can
     be written on disk (i.e. remove/replace invalid
     characters and encode in the native language) and
     trim filename if greater than max_length (0 = no limit).
+
+    If use_ascii is True, don't encode in the native language,
+    but use only characters from the ASCII character set.
     """
     if max_length > 0 and len(filename) > max_length:
         log('Limiting file/folder name "%s" to %d characters.', filename, max_length)
         filename = filename[:max_length]
 
     global encoding
-    return re.sub('[/|?*<>:+\[\]\"\\\]', '_', filename.strip().encode(encoding, 'ignore'))
+    if use_ascii:
+        e = 'ascii'
+    else:
+        e = encoding
+    return re.sub('[/|?*<>:+\[\]\"\\\]', '_', filename.strip().encode(e, 'ignore'))
 
 
 def find_mount_point(directory):
